

; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley
;
; Script Function:
;   MultiRunConfig - MultiRun Configuration Utility
;

; Includes
#include <Constants.au3>
#include <GUIConstants.au3>
#include <GUIList.au3>

#include "MultiRunDefs.au3"
#include "MultiRunFunc.au3"


;
; Global values
;

; Title Information
$TitleBar = "MultiRunConfig"
$TitleBar2 = "MultiRun Config Utility v1.2, (c) 2007 Michael W. Bayley"
$TitleMain = " - Main"
$TitleProfile = " - Profile Configuration"
$TitleProgram = " - Program Configuration"
$TitlePBrowse = " - Browse for Program"
$TitleDBrowse = " - Browse for Start Directory"
$TitleTBrowse = " - Browse for Title"
$TitleAuthor = "By Michael W. Bayley"

; Ini/Mrn Option Values
$IniFile = ""
$IniWorkFile = ""
$IniWorkFileExtension = ".$$$"

; Log Values
$Log = ""
 
; Program ID
$Tag = ""
$Major = 0
$Minor = 0

; Program Values
$Profiles = 0
$Programs = 0
$SectionID = 0
$Items = 0
$Item = ""
$Item2 = ""

; Dialog Values
$MainDlg = -1
$MainList = -1
$MainCheck1 = -1
$MainCheck2 = -1
$MainCheck3 = -1
$MainInput = -1
$MainOrgWarnProfile = 1
$MainOrgWarnProgram = 1
$MainOrgUseRelativePaths = 1
$MainFlag = 1
$MainFlag2 = 1

$MainDone = -1
$MainCancel = -1
$MainAutorun = -1
$MainAdd = -1
$MainEdit = -1
$MainDelete = -1

$TextProfiles = "Profiles"
$TextLogName = "Log File"
$TextWarnProfile = "Warn on Profile Delete"
$TextWarnProgram = "Warn on Program Delete"
$TextUseRelativePaths = "Use Relative Paths"

$TextProfile = "Profile"
$TextPrograms = "Programs"
$TextListProgram = "Program                             "
$TextListClose = "Wait   "

$TextEPTitle1 = "Profile: "
$TextEPTitle2 = "  Program Entry "
$TextEPTitle3 = " of "

$TextProgGroup = "Program Information"
$TextProgName = "Program Executable:"
$TextProgDir = "Starting Directory:"
$TextProgParms = "Parameters:"
$TextProgMode = "Run Mode"
$TextProgModeNormal = "Normal Window"
$TextProgModeHidden = "Hidden Window"
$TextProgModeMin = "Minimized"
$TextProgModeMax = "Maximized"
$TextProgSingle = "Single Instance"

$TextTitleGroup = "Title Match Information"
$TextTitle1 = "Title Text 1:"
$TextTitle2 = "Title Text 2:"
$TextTitleMatch = "Match Mode"
$TextTitleModeNone = "Disabled"
$TextTitleModeExact = "Exact Match"
$TextTitleModeStart = "Starts With"
$TextTitleModeContain = "Contains"
$TextTitleModeRegexp = "Starts/Ends With"

$TextWindowPosGroup = "Window Positioning Information"
$TextWinXPos = "X Position:"
$TextWinYPos = "Y Position:"
$TextWinWidth = "Width:"
$TextWinHeight = "Height:"
$TextPosDefault = "Default"
$TextPosCenter = "Centered"
$TextPosition = "Absolute:"

$TextKeyGroup = "Text Send Information"
$TextKeys = "Key Sequence"

$TextWaitGroup = "Program Delay Information"
$TextWait1 = "Until Program Closes with Timeout:"
$TextWait2 = "Second(s)"

$TextTitles = "Window Titles"

$TextDone = "Done"
$TextCancel = "Cancel"
$TextAutorun = "Create Autorun"
$TextAdd = "Add"
$TextEdit = "Edit"
$TextDelete = "Delete"
$TextBrowse = "..."
$TextPath = "P"
$TextTest = "Test"
$TextOk = "OK"

$UpArrow = @ScriptDir
If StringRight($UpArrow, 1) <> "\" Then
	$UpArrow &= "\"
EndIf
$DownArrow = $UpArrow
$UpArrow &= "UpArrow.bmp"
$DownArrow &= "DownArrow.bmp"

$SepChar = Chr(187)
$NotDone = 0
$Cancelled = 0
$Msg = 0
$RetCode = 0


;
; Main Code
;
; Entry:		<None>
; Return:		0 - No errors
;			    1 - Fatal error encountered
; Procedure:	1. If no parameters then use default config file.
;			    2. If one parameter then use it as the config file.
;			    3. If too many parameters then report error.
;			    4. If file does not exists, ask user to create it.
;			    5. Validate file.
;			    6. Backup the config file.
;			    7. Build main dialog
;			    8. Process user commands..
;			    9. If ok then replace config file.
;			    10. Exit all done.

AutoItSetOption("GUIDataSeparatorChar", $SepChar)
; 1. If no parameters then use default config file.
If $CmdLine[0] == 0 Then
	$IniFile = @ScriptDir
	If StringRight($IniFile, 1) == "\" Then
		$IniFile = $IniFile & $IniFileName
	Else
		$IniFile = $IniFile & "\" & $IniFileName
	EndIf

	
; 2, If one parameter then use it as the config file.
ElseIf $CmdLine[0] == 1 Then
	$IniFile = @ScriptDir
	If StringRight($IniFile, 1) == "\" Then
		$IniFile = $IniFile & $CmdLine[1]
	Else
		$IniFile = $IniFile & "\" & $CmdLine[1]
	EndIf


; 3. If too many parameters then report error.
Else
	$msg = "This program is used to edit the profiles for MultiRun.  The" & @CRLF
	$msg &= "command line syntax is as follows:" & @CRLF & @CRLF
	$msg &= "        MultRunConfig.exe <ConfigName>" & @CRLF & @CRLF
	$msg &= "If the 'ConfigName' parameter is not supplied then the default" & @CRLF
	$msg &= "file 'MultiRun.mrn' will be used." & @CRLF & @CRLF
	MsgBox(0, $TitleBar2, $msg)
	Exit(1)
EndIf


; 4. If file does not exists, ask user to create it.
If FileExists($IniFile) <> 1 Then
	If MsgBox(4, $TitleBar2, "File does not exists.  Create it?") <> 6 Then Exit(1)
   
	$msg = FileOpen($IniFile, 10)
	If $msg == -1 Then
		MsgBox(0, $TitleBar2, "Fatal Error - Can't create " & $IniFile & "!")
		Exit(1)
	EndIf
	FileClose($msg)

	$Items = IniWrite($IniFile, $IniSection, $IniSectionID, $IniMainSectionID)
	$Items += IniWrite($IniFile, $IniSection, $IniProgramID, $ProgramID)
	$Items += IniWrite($IniFile, $IniSection, $IniMajor, $ProgramMajor)
	$Items += IniWrite($IniFile, $IniSection, $IniMinor, $ProgramMinor)
	$Items += IniWrite($IniFile, $IniSection, $IniLog, "")

	$Items += IniWrite($IniFile, $IniProfilesSection, $IniSectionID, $IniProfilesID)
	$Items += IniWrite($IniFile, $IniProfilesSection, $IniItems, 1)
	$Items += IniWrite($IniFile, $IniProfilesSection, $IniItem & "1", $IniDefaultProfile)

	$Items += IniWrite($IniFile, $IniDefaultSection, $IniSectionID, $IniProfilesID)
	$Items += IniWrite($IniFile, $IniDefaultSection, $IniItems, 0)

	If $Items <> 10 Then
		MsgBox(0, $TitleBar2, "Error - One or more errors occured while initializing '" & $IniFile & "'")
		Exit(1)
	EndIf
EndIf


; 5. Validate file.
$SectionID = Number(IniRead($IniFile, $IniSection, $IniSectionID, 0))
$Tag = IniRead($IniFile, $IniSection, $IniProgramID, "")
$Major = Number(IniRead($IniFile, $IniSection, $IniMajor, $Major))
$Minor = Number(IniRead($IniFile, $IniSection, $IniMinor, $Minor))

If $Tag <> $ProgramID Then
	MsgBox(0, $TitleBar2, "Error - '" & $IniFile & "' is not a valid MultiRun file.")
	Exit(1)
EndIf

If $Major <> $ProgramMajor OR $Minor <> $ProgramMinor Then
	MsgBox(0, $TitleBar2, "Error - '" & $IniFile & "' is a different version.")
	Exit(1)
EndIf

If $SectionID <> $IniMainSectionID Then
	If MsgBox(260, $TitleBar2, "Error - '" & $IniFile & "' appears to be corrupt. Attempt to fix it?") <> 6 Then
		Exit(1)
	EndIf
	
	If FileWrite($IniFile, $IniSection, $IniSectionID, $IniMainSectionID) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure attempting to repair '" & $IniFile & "'")
		Exit(1)
	EndIf
EndIf


; 6. Backup the config file.
$IniWorkFile = $IniFile & $IniWorkFileExtension
If FileCopy($IniFile, $IniWorkFile, 1) <> 1 Then
	MsgBox(0, $TitleBar2, "Error - Unable to create work file '" & $IniWorkFile & "'")
	Exit(1)
EndIf


; 7. Build main dialog
; Main data.
$Log = IniRead($IniWorkFile, $IniSection, $IniLog, "")
$SectionID = Number(IniRead($IniWorkFile, $IniProfilesSection, $IniSectionID, 0))
If $SectionID <> $IniProfilesID Then
	If MsgBox(260, $TitleBar2, "Error - '" & $IniWorkFile & "' appears to be corrupt. Attempt to fix it?") <> 6 Then
		Exit(1)
	EndIf
	
	If FileWrite($IniWorkFile, $IniProfilesSection, $IniSectionID, $IniProfilesID) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure attempting to repair '" & $IniWorkFile & "'")
		Exit(1)
	EndIf
EndIf
$Items = Number(IniRead($IniWorkFile, $IniProfilesSection, $IniItems, 0))

; Dialog components
$MainDialog = GUICreate($TitleBar & $TitleMain, 280, 420)
GUICtrlCreateLabel($TextProfiles, 20, 10, 240, 20)
$MainList = GUICtrlCreateList("", 20, 30, 240, 140)
$MainAdd = GUICtrlCreateButton($TextAdd, 10, 180, 80, 30)
$MainEdit = GUICtrlCreateButton($TextEdit, 100, 180, 80, 30)
$MainDelete = GUICtrlCreateButton($TextDelete, 190, 180, 80, 30)
GUICtrlCreateLabel($TextLogName, 20, 230, 240, 20)
$MainInput = GUICtrlCreateInput("", 20, 250, 240, 20)
$MainCheck1 = GUICtrlCreateCheckbox($TextWarnProfile, 10, 285, 130, 20)
$MainCheck2 = GUICtrlCreateCheckbox($TextWarnProgram, 140, 285, 135, 20)
$MainCheck3 = GUICtrlCreateCheckbox($TextUseRelativePaths, 90, 305, 135, 20)
$MainAutorun = GUICtrlCreateButton($TextAutorun, 50, 335, 180, 20)
$MainDone = GUICtrlCreateButton($TextDone, 50, 370, 80, 30)
$MainCancel = GUICtrlCreateButton($TextCancel, 150, 370, 80, 30)

; Populate dialog
$MainOrgWarnProfile = Number(IniRead($IniWorkFile, $IniSection, $IniWarnProfile, 1))
If $MainOrgWarnProfile Then
	GUICtrlSetState($MainCheck1, $GUI_CHECKED)
Else
	GUICtrlSetState($MainCheck1, $GUI_UNCHECKED)
EndIf
$MainOrgWarnProgram = Number(IniRead($IniWorkFile, $IniSection, $IniWarnProgram, 1))
If $MainOrgWarnProgram Then
	GUICtrlSetState($MainCheck2, $GUI_CHECKED)
Else
	GUICtrlSetState($MainCheck2, $GUI_UNCHECKED)
EndIf
$MainOrgUseRelativePaths = Number(IniRead($IniWorkFile, $IniSection, $IniUseRelativePaths, 1))
If $MainOrgUseRelativePaths Then
	GUICtrlSetState($MainCheck3, $GUI_CHECKED)
Else
	GUICtrlSetState($MainCheck3, $GUI_UNCHECKED)
EndIf
$Items = BuildProfilesList($MainList, $IniDefaultProfile)
GUICtrlSetData($MainInput, $Log)
GUICtrlSetState($MainDone, $GUI_DEFBUTTON)
GUISetState(@SW_SHOW, $MainDialog)


; 8. Process user commands..
$Cancelled = 0
$NotDone = 1
While $NotDone
	$Msg = GUIGetMsg()

    Select
        Case $Msg == $GUI_EVENT_CLOSE
			; Close dialog
			$Cancelled = 1
            $NotDone = 0
        Case $Msg == $MainDone
			; Done button
			$NotDone = 0
		Case $Msg == $MainCancel
			; Cancel button
			If MsgBox(260, $TitleBar2, "Warning - All changes will be lost.  Sure you want to cancel?" & @CRLF) == 6 Then
				$Cancelled = 1
				$NotDone = 0
			EndIf
		Case $Msg == $MainAutorun
			CreateAutorun()
        Case $Msg == $MainAdd
			; Add profile button
			GUISetState(@SW_DISABLE, $MainDialog)
			$Item = AddProfileToIni()
			If @error Then
				$Items = BuildProfilesList($MainList, $IniDefaultProfile)
			Else
				If GUICtrlRead($MainCheck2) == $GUI_CHECKED Then
					$MainFlag = 1
				Else
					$MainFlag = 0
				EndIf
				If GUICtrlRead($MainCheck3) == $GUI_CHECKED Then
					$MainFlag2 = 1
				Else
					$MainFlag2 = 0
				EndIf
				$Item = EditProfileDialog($Item, $MainFlag, $MainFlag2)
				$Items = BuildProfilesList($MainList, $Item)
			EndIf
			GUISetState(@SW_ENABLE)
			GUISetState(@SW_HIDE)
			GUISetState(@SW_SHOW)
        Case $Msg == $MainEdit
			; Edit profile button
			$Item = GUICtrlRead($MainList)
			If GUICtrlRead($MainCheck2) == $GUI_CHECKED Then
				$MainFlag = 1
			Else
				$MainFlag = 0
			EndIf
			If GUICtrlRead($MainCheck3) == $GUI_CHECKED Then
				$MainFlag2 = 1
			Else
				$MainFlag2 = 0
			EndIf
			If StringLen($Item) == 0 Then
				MsgBox(0, $TitleBar2, "Error - No profile selected")
			Else
				GUISetState(@SW_DISABLE, $MainDialog)
				$Item = EditProfileDialog($Item, $MainFlag, $MainFlag2)
				$Items = BuildProfilesList($MainList, $Item)
				GUISetState(@SW_ENABLE, $MainDialog)
				GUISetState(@SW_HIDE, $MainDialog)
				GUISetState(@SW_SHOW, $MainDialog)
			EndIf
        Case $Msg == $MainDelete
			; Delete profile button
			$Item = GUICtrlRead($MainList)
			If StringLen($Item) == 0 Then
				MsgBox(0, $TitleBar2, "Error - No profile selected")
			ElseIf StringUpper($Item) == StringUpper($IniDefaultProfile) Then
				MsgBox(0, $TitleBar2, "Error - You can't delete the '(Default)' profile")
			Else
				If $MainWarnProfile Then
					$Item2 = GUICtrlRead($MainList)
					If MsgBox(260, $TitleBar2, "Warning - Are you sure you want to delete profile '" & $Item2 & "'?") == 6 Then
						DeleteProfileFromIni($Item)
						$Items = BuildProfilesList($MainList, $IniDefaultProfile)
					EndIf
				Else
					DeleteProfileFromIni($Item)
					$Items = BuildProfilesList($MainList, $IniDefaultProfile)
				EndIf
			EndIf
	EndSelect
WEnd


; 9. If ok then replace config file.
$RetCode = 0
If $Cancelled == 0 Then
	If GUICtrlRead($MainCheck1) == $GUI_CHECKED Then
		$Item = 1
	Else
		$Item = 0
	EndIf
	If $MainOrgWarnProfile <> $Item Then
		$MainOrgWarnProfile = $Item
		If IniWrite($IniWorkFile, $IniSection, $IniWarnProfile, $MainOrgWarnProfile) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure saving profile warning to '" & $IniWorkFile & "' as '" & $IniFIle & "'")
		EndIf
	EndIf

	If GUICtrlRead($MainCheck2) == $GUI_CHECKED Then
		$Item = 1
	Else
		$Item = 0
	EndIf
	If $MainOrgWarnProgram <> $Item Then
		$MainOrgWarnProgram = $Item
		If IniWrite($IniWorkFile, $IniSection, $IniWarnProgram, $MainOrgWarnProgram) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure saving program warning to '" & $IniWorkFile & "' as '" & $IniFIle & "'")
		EndIf
	EndIf

	If GUICtrlRead($MainCheck3) == $GUI_CHECKED Then
		$Item = 1
	Else
		$Item = 0
	EndIf
	If $MainOrgUseRelativePaths <> $Item Then
		$MainOrgUseRelativePaths = $Item
		If IniWrite($IniWorkFile, $IniSection, $IniUseRelativePaths, $MainOrgUseRelativePaths) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure saving relative paths flag to '" & $IniWorkFile & "' as '" & $IniFIle & "'")
		EndIf
	EndIf

	If FileCopy($IniWorkFile, $IniFile, 1) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure saving work file '" & $IniWorkFile & "' as '" & $IniFIle & "'")
		$RetCode = 1
	EndIf
EndIf
FileDelete($IniWorkFile)
GUIDelete($MainDialog)


; 10. Exit all done.
Exit($RetCode)


;
; EditProfileDialog($profile, $flag, $flag2)
;
; Entry:	    $profile	Profile to edit
;			    $flag		Warn on progra, delete
;			    $flag2		Use relative paths flag
; Return:       $profile	New name of profile
; Procedure:	1. Build the dialog.
;			    2. Populate the dialog.
;			    3. Process user commands.
;			    4. Update profile name if needed.
;			    5. Return new profile name.
;
Func	EditProfileDialog($profile, $flag, $flag2)
	Local	$dlg
	Local	$input1, $list1
	Local	$button1
	Local	$button3, $button4, $button5
	Local	$button6, $button7
	Local	$notDone, $msg
	Local	$items, $item, $item2, $index
	Local	$newProfile

	; 1. Build main dialog
	$dlg = GUICreate($TitleBar & $TitleProfile, 280, 340)
	GUICtrlCreateLabel($TextProfile, 20, 10, 240, 20)
	$input1 = GUICtrlCreateInput("", 20, 30, 240, 20)
	GUICtrlCreateLabel($TextPrograms, 50, 60, 210, 20)
	$list1 = GUICtrlCreateList("", 50, 80, 210, 140, $WS_VSCROLL)
	$button6 = GUICtrlCreateButton("", 20, 120, 20, 20, $BS_BITMAP)
	$button7 = GUICtrlCreateButton("", 20, 150, 20, 20, $BS_BITMAP)
	$button3 = GUICtrlCreateButton($TextAdd, 10, 240, 80, 30)
	$button4 = GUICtrlCreateButton($TextEdit, 100, 240, 80, 30)
	$button5 = GUICtrlCreateButton($TextDelete, 190, 240, 80, 30)
	$button1 = GUICtrlCreateButton($TextDone, 100, 290, 80, 30)

	
	; 2. Populate dialog
	$newProfile = $profile
	GUICtrlSetData($input1, $newProfile)
	If $newProfile == $IniDefaultProfile Then
		GUICtrlSetState($input1, $GUI_DISABLE)
	EndIf
	$items = BuildProgramsList($list1, $newProfile, 0)
	GUICtrlSetImage($button6, $UpArrow)
	GUICtrlSetImage($button7, $DownArrow)
	GUICtrlSetState($button1, $GUI_DEFBUTTON)
	GUISetState(@SW_SHOW, $dlg)


	; 3. Process user commands..
	$NotDone = 1
	While $notDone
		$msg = GUIGetMsg()

		Select
			Case $msg == $button1
				; Done button
				$notDone = 0
			Case $msg == $button3
				; Add program button
				GUISetState(@SW_DISABLE, $dlg)
				$index = AddProgramEntryToIni($newProfile)
				$item2 = EditProgramDialog($newProfile, $index, $index, $flag2)
				If $item2 Then
					$items = BuildProgramsList($list1, $newProfile, $index - 1)
				    MsgBox(0, "WORK HERE", "Ret = " & $items)
				Else
					DeleteProgramEntryFromIni($newProfile, $index)
				EndIf
				GUISetState(@SW_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $Msg == $button4
				; Edit program button
				$item = GUICtrlRead($list1)
				If StringLen($item) == 0 Then
					MsgBox(0, $TitleBar2, "Error - No program selected.")
				Else
					$index = _GUICtrlListSelectedIndex($list1) + 1
					GUISetState(@SW_DISABLE, $dlg)
					$item2 = EditProgramDialog($newProfile, $index, $items, $flag2)
					GUISetState(@SW_ENABLE, $dlg)
					GUISetState(@SW_HIDE, $dlg)
					GUISetState(@SW_SHOW, $dlg)
				EndIf
			Case $Msg == $button5
				; Delete program button
				$item = GUICtrlRead($list1)
				If StringLen($item) == 0 Then
					MsgBox(0, $TitleBar2, "Error - No program selected")
				Else
					If $flag Then
						If MsgBox(260, $TitleBar2, "Warning - Are you sure you want to delete program '" & $Item & "'?") == 6 Then
            				$item = _GUICtrlListSelectedIndex($list1) + 1
							DeleteProgramEntryFromIni($newProfile, $item)
							$Items = BuildProgramsList($list1, $newProfile, 0)
						EndIf
					Else
        				$item = _GUICtrlListSelectedIndex($list1) + 1
						DeleteProgramEntryFromIni($newProfile, $item)
						$Items = BuildProgramsList($list1, $newProfile, 0)
					EndIf
				EndIf
			Case $Msg == $button6
				; Move program up button
				$item = _GUICtrlListSelectedIndex($list1) + 1
				If $item > 1 Then
					SwapProgramEntriesInIni($newProfile, $item - 1, $item)
					$items = BuildProgramsList($list1, $newProfile, $item - 2)
				EndIf
				GUICtrlSetState($list1, $GUI_FOCUS)
			Case $Msg == $button7
				; Move program down button
				$item = _GUICtrlListSelectedIndex($list1) + 1
				If $item < $items Then
					SwapProgramEntriesInIni($newProfile, $item, $item + 1)
					$items = BuildProgramsList($list1, $newProfile, $item)
				EndIf
				GUICtrlSetState($list1, $GUI_FOCUS)
		EndSelect
	WEnd


	; 4. Update profile name if needed.
	$newProfile = GUICtrlRead($Input1)
	If $newProfile <> $profile Then
		RenameProfileInIni($profile, $newProfile)
	EndIf


	; 5. Return new profile name.
	GUIDelete($dlg)
	Return($newProfile)
EndFunc



; EditProgramDialog($profile, entryNo, $total, $flag)
;
; Entry:	   $profile	Profile to edit
;			    $entryNo	Program entry number
;			    $total		Total items in profile
;			    $flag		Use relative paths flag
; Return:		1 if all ok
;			    0 if user abort
; Procedure:	1.  Build the dialog.
;			    2. Populate the dialog.
;			    3. Process user commands.
;			    4. Update program data if needed.
;			    5. Return all done
;
Func	EditProgramDialog($profile, $entryNo, $total, $flag)
	Local	$dlg
	Local	$label1

	Local	$input1a, $input1b, $input1c
	Local	$radio1a, $radio1b, $radio1c, $radio1d
	Local	$check1
	Local	$button1a, $button1b, $button1c, $button1d

	Local	$input2a, $input2b
	Local	$radio2a, $radio2b, $radio2c, $radio2d, $radio2e
	Local	$button2a, $button2b, $button2c
	Local	$titleDisable, $titleEnable

	Local	$radio3a, $radio3b, $radio3c, $input3a
	Local	$radio3d, $radio3e, $radio3f, $input3d
	Local	$radio3g, $radio3h, $radio3i, $input3g
	Local	$radio3j, $radio3k, $radio3l, $input3j
	Local	$button3a
	Local	$positionDisable, $positionEnable

	Local	$input4a, $button4a
	Local	$keysDisable, $keysEnable
	
	Local	$radio5a, $radio5b
	Local	$input5a, $input5b
	Local	$button5a

	Local	$button1, $button2
	Local	$section
	Local	$notDone, $msg, $cancelled
	Local	$items, $item, $item2, $item3, $item4, $item5

	; 1. Build main dialog
	$dlg = GUICreate($TitleBar & $TitleProgram, 602, 480)
	$label1 = GUICtrlCreateLabel($TextEPTitle1 & $profile & $TextEPTitle2 & $entryNo & $TextEPTitle3 & $total, 170, 8, 300, 30)
	GUICtrlSetFont($label1, 10, 600)
	GUICtrlCreateGroup($TextProgGroup, 15, 30, 280, 250)
	GUICtrlCreateLabel($TextProgName, 30, 50, 220, 20)
	$input1a = GUICtrlCreateInput("", 30, 65, 220, 20)
	GUICtrlCreateLabel($TextProgDir, 30, 95, 220, 20)
	$input1b = GUICtrlCreateInput("", 30, 110, 220, 20)
	GUICtrlCreateLabel($TextProgParms, 30, 140, 220, 20)
	$input1c = GUICtrlCreateInput("", 30, 155, 220, 20)
	GUICtrlCreateLabel($TextProgMode, 30, 185, 220, 20)
	GUIStartGroup($dlg)
	$radio1a = GUICtrlCreateRadio($TextProgModeNormal, 30, 200, 100, 20)
	$radio1b = GUICtrlCreateRadio($TextProgModeHidden, 30, 220, 100, 20)
	$radio1c = GUICtrlCreateRadio($TextProgModeMin, 170, 200, 100, 20)
	$radio1d = GUICtrlCreateRadio($TextProgModeMax, 170, 220, 100, 20)
	$check1 = GUICtrlCreateCheckbox($TextProgSingle, 30, 250, 100, 20)
	$button1a = GUICtrlCreateButton($TextBrowse, 260, 65, 20, 20)
	$button1b = GUICtrlCreateButton($TextBrowse, 260, 110, 20, 20)
	$button1c = GUICtrlCreateButton($TextPath, 260, 155, 20, 20)
	$button1d = GUICtrlCreateButton($TextTest, 220, 250, 60, 20)

	GUICtrlCreateGroup($TextTitleGroup, 310, 30, 280, 200)
	GUICtrlCreateLabel($TextTitle1, 325, 50, 220, 20)
	$input2a = GUICtrlCreateInput("", 325, 65, 220, 20)
	GUICtrlCreateLabel($TextTitle2, 325, 95, 220, 20)
	$input2b = GUICtrlCreateInput("", 325, 110, 220, 20)
	GUICtrlCreateLabel($TextTitleMatch, 325, 140, 220, 20)
	GUIStartGroup($dlg)
	$radio2a = GUICtrlCreateRadio($TextTitleModeNone, 325, 155, 100, 20)
	$radio2b = GUICtrlCreateRadio($TextTitleModeExact, 325, 175, 100, 20)
	$radio2c = GUICtrlCreateRadio($TextTitleModeStart, 325, 195, 100, 20)
	$radio2d = GUICtrlCreateRadio($TextTitleModeContain, 465, 155, 100, 20)
	$radio2e = GUICtrlCreateRadio($TextTitleModeRegexp, 465, 175, 100, 20)
	$button2a = GUICtrlCreateButton($TextBrowse, 555, 65, 20, 20)
	$button2b = GUICtrlCreateButton($TextBrowse, 555, 110, 20, 20)
	$button2c = GUICtrlCreateButton($TextTest, 515, 200, 60, 20)

	GUICtrlCreateGroup($TextWindowPosGroup, 310, 240, 280, 200)
	GUICtrlCreateLabel($TextWinXPos, 325, 260, 80, 20)
	GUIStartGroup($dlg)
	$radio3a = GUICtrlCreateRadio($TextPosDefault, 323, 275, 70, 20)
	$radio3b = GUICtrlCreateRadio($TextPosCenter, 396, 275, 70, 20)
	$radio3c = GUICtrlCreateRadio($TextPosition, 474, 275, 70, 20)
	$input3a = GUICtrlCreateInput("", 541, 274, 33, 20)
	GUICtrlCreateLabel($TextWinYPos, 325, 295, 80, 20)
	GUIStartGroup($dlg)
	$radio3d = GUICtrlCreateRadio($TextPosDefault, 323, 310, 70, 20)
	$radio3e = GUICtrlCreateRadio($TextPosCenter, 396, 310, 70, 20)
	$radio3f = GUICtrlCreateRadio($TextPosition, 474, 310, 70, 20)
	$input3d = GUICtrlCreateInput("", 541, 309, 33, 20)
	GUICtrlCreateLabel($TextWinWidth, 325, 330, 80, 20)
	GUIStartGroup($dlg)
	$radio3g = GUICtrlCreateRadio($TextPosDefault, 323, 345, 70, 20)
	$radio3h = GUICtrlCreateRadio($TextPosCenter, 396, 345, 70, 20)
	$radio3i = GUICtrlCreateRadio($TextPosition, 474, 345, 70, 20)
	$input3g = GUICtrlCreateInput("", 541, 344, 33, 20)
	GUICtrlCreateLabel($TextWinHeight, 325, 365, 80, 20)
	GUIStartGroup($dlg)
	$radio3j = GUICtrlCreateRadio($TextPosDefault, 323, 380, 70, 20)
	$radio3k = GUICtrlCreateRadio($TextPosCenter, 396, 380, 70, 20)
	$radio3l = GUICtrlCreateRadio($TextPosition, 474, 380, 70, 20)
	$input3j = GUICtrlCreateInput("", 541, 379, 33, 20)
	$button3a = GUICtrlCreateButton($TextTest, 515, 410, 60, 20)

	GUICtrlCreateGroup($TextKeyGroup, 15, 290, 280, 95)
	GUICtrlCreateLabel($TextKeys, 30, 310, 220, 20)
	$input4a = GUICtrlCreateInput("", 30, 325, 250, 20)
	$button4a = GUICtrlCreateButton($TextTest, 220, 355, 60, 20)

	GUICtrlCreateGroup($TextWaitGroup, 15, 395, 280, 75)
	$radio5a = GUICtrlCreateRadio($TextWait1, 30, 414, 190, 20)
	$input5a = GUICtrlCreateInput("", 220, 415, 60, 20)
	$radio5b = GUICtrlCreateRadio("", 30, 435, 15, 20)
	$input5b = GUICtrlCreateInput("", 47, 435, 60, 20)
	GUICtrlCreateLabel($TextWait2, 115, 439, 100, 20)
	$button5a = GUICtrlCreateButton($TextTest, 220, 445, 60, 20)

	$button1 = GUICtrlCreateButton($TextDone, 360, 445, 80, 30)
	$button2 = GUICtrlCreateButton($TextCancel, 460, 445, 80, 30)


	; 2. Populate dialog
	$section = $profile & "-" & StringRight("00" & $entryNo, 2)
	$titleDisable = 0
	$titleEnable = 0
	$positionDisable = 0
	$positionEnable = 0
	$keysDisable = 0
	$keysEnable = 0

	$item = IniRead($IniWorkFile, $section, $IniRunProgram, "")
	GUICtrlSetData($input1a, $item)
	$item = IniRead($IniWorkFile, $section, $IniRunDir, "")
	GUICtrlSetData($input1b, $item)
	$item = IniRead($IniWorkFile, $section, $IniRunParams, "")
	GUICtrlSetData($input1c, $item)
	If $flag == 0 Then
		GUICtrlSetState($button1c, $GUI_DISABLE)
	EndIf
	$item = Number(IniRead($IniWorkFile, $section, $IniRunProgramMode, 0))
	Select
		Case $item == $IniRunProgramModeNormal
			GuiCtrlSetState($radio1a, $GUI_CHECKED)
		Case $item == $IniRunProgramModeHide
			GuiCtrlSetState($radio1b, $GUI_CHECKED)
			$titleDisable = 1
			$positionDisable = 1
			$keysDisable = 1
		Case $item == $IniRunProgramModeMin
			GuiCtrlSetState($radio1c, $GUI_CHECKED)
			$positionDisable = 1
		Case $item == $IniRunProgramModeMax
			GuiCtrlSetState($radio1d, $GUI_CHECKED)
			$positionDisable = 1
	EndSelect
	$item = Number(IniRead($IniWorkFile, $section, $IniRunSingleInstance, 0))
	If $item Then
		GUICtrlSetState($check1, $GUI_CHECKED)
	Else
		GUICtrlSetState($check1, $GUI_UNCHECKED)
	EndIf

	$item = IniRead($IniWorkFile, $section, $IniRunTitle1, "")
	GUICtrlSetData($input2a, $item)
	$item = IniRead($IniWorkFile, $section, $IniRunTitle2, "")
	GUICtrlSetData($input2b, $item)
	$item = Number(IniRead($IniWorkFile, $section, $IniRunTitleMode, 0))
	Select
		Case $item == $IniRunTitleModeNone
			GuiCtrlSetState($radio2a, $GUI_CHECKED)
			$titleDisable = 1
			$positionDisable = 1
			$keysDisable = 1
		Case $item == $IniRunTitleModeExact
			GuiCtrlSetState($radio2b, $GUI_CHECKED)
		Case $item == $IniRunTitleModeStart
			GuiCtrlSetState($radio2c, $GUI_CHECKED)
		Case $item == $IniRunTitleModeContains
			GuiCtrlSetState($radio2d, $GUI_CHECKED)
		Case $item == $IniRunTitleModeRegexp
			GuiCtrlSetState($radio2e, $GUI_CHECKED)
	EndSelect

	$item = Number(IniRead($IniWorkFile, $section, $IniRunXPos, 0))
	If $item == $IniRunDefault Then
		GuiCtrlSetState($radio3a, $GUI_CHECKED)
		GUICtrlSetData($input3a, "")
		GUICtrlSetState($input3a, $GUI_DISABLE)
	ElseIf $item == $IniRunCenter Then
		GuiCtrlSetState($radio3b, $GUI_CHECKED)
		GUICtrlSetData($input3a, "")
		GUICtrlSetState($input3a, $GUI_DISABLE)
	Else
		GuiCtrlSetState($radio3c, $GUI_CHECKED)
		GUICtrlSetData($input3a, $item)
		GUICtrlSetState($input3a, $GUI_ENABLE)
	EndIf
	$item = Number(IniRead($IniWorkFile, $section, $IniRunYPos, 0))
	If $item == $IniRunDefault Then
		GuiCtrlSetState($radio3d, $GUI_CHECKED)
		GUICtrlSetData($input3d, "")
		GUICtrlSetState($input3d, $GUI_DISABLE)
	ElseIf $item == $IniRunCenter Then
		GuiCtrlSetState($radio3e, $GUI_CHECKED)
		GUICtrlSetData($input3d, "")
		GUICtrlSetState($input3d, $GUI_DISABLE)
	Else
		GuiCtrlSetState($radio3f, $GUI_CHECKED)
		GUICtrlSetData($input3d, $item)
		GUICtrlSetState($input3d, $GUI_ENABLE)
	EndIf
	$item = Number(IniRead($IniWorkFile, $section, $IniRunWidth, 0))
	If $item == $IniRunDefault Then
		GuiCtrlSetState($radio3g, $GUI_CHECKED)
		GUICtrlSetData($input3g, "")
		GUICtrlSetState($input3g, $GUI_DISABLE)
	ElseIf $item == $IniRunCenter Then
		GuiCtrlSetState($radio3h, $GUI_CHECKED)
		GUICtrlSetData($input3g, "")
		GUICtrlSetState($input3g, $GUI_DISABLE)
	Else
		GuiCtrlSetState($radio3i, $GUI_CHECKED)
		GUICtrlSetData($input3g, $item)
		GUICtrlSetState($input3g, $GUI_ENABLE)
	EndIf
	$item = Number(IniRead($IniWorkFile, $section, $IniRunHeight, 0))
	If $item == $IniRunDefault Then
		GuiCtrlSetState($radio3j, $GUI_CHECKED)
		GUICtrlSetData($input3j, "")
		GUICtrlSetState($input3j, $GUI_DISABLE)
	ElseIf $item == $IniRunCenter Then
		GuiCtrlSetState($radio3k, $GUI_CHECKED)
		GUICtrlSetData($input3j, "")
		GUICtrlSetState($input3j, $GUI_DISABLE)
	Else
		GuiCtrlSetState($radio3l, $GUI_CHECKED)
		GUICtrlSetData($input3j, $item)
		GUICtrlSetState($input3j, $GUI_ENABLE)
	EndIf

	$item = IniRead($IniWorkFile, $section, $IniRunKeys, "")
	GUICtrlSetData($input4a, $item)

	$item = Number(IniRead($IniWorkFile, $section, $IniRunWait, 0))
	If $item == $IniRunWaitClose Then
		GUICtrlSetState($radio5a, $GUI_CHECKED)
		$item = Number(IniRead($IniWorkFile, $section, $IniRunTimeout, 0))
		GUICtrlSetData($input5a, $item)
		GUICtrlSetData($input5b, "")
		GUICtrlSetState($input5b, $GUI_DISABLE)
	Else
		GUICtrlSetState($radio5b, $GUI_CHECKED)
		GUICtrlSetData($input5b, $item)
		GUICtrlSetData($input5a, "")
		GUICtrlSetState($input5a, $GUI_DISABLE)
	EndIf

	GUICtrlSetState($button1, $GUI_DEFBUTTON)
	GUISetState(@SW_SHOW, $dlg)


	; 3. Process user commands..
	$NotDone = 1
	While $notDone
		If $titleDisable Then
			If GUICtrlRead($radio1b) == $GUI_CHECKED Then
				GUICtrlSetState($radio2a, $GUI_DISABLE)
				GUICtrlSetState($radio2b, $GUI_DISABLE)
				GUICtrlSetState($radio2c, $GUI_DISABLE)
				GUICtrlSetState($radio2d, $GUI_DISABLE)
				GUICtrlSetState($radio2e, $GUI_DISABLE)
			EndIf
			GUICtrlSetState($input2a, $GUI_DISABLE)
			GUICtrlSetState($input2b, $GUI_DISABLE)
			GUICtrlSetState($button2a, $GUI_DISABLE)
			GUICtrlSetState($button2b, $GUI_DISABLE)
			GUICtrlSetState($button2c, $GUI_DISABLE)
			$titleDisable = 0
		EndIf

		If $titleEnable Then
			If GUICtrlRead($radio1d) == $GUI_UNCHECKED Then
				GUICtrlSetState($radio2a, $GUI_ENABLE)
				GUICtrlSetState($radio2b, $GUI_ENABLE)
				GUICtrlSetState($radio2c, $GUI_ENABLE)
				GUICtrlSetState($radio2d, $GUI_ENABLE)
				GUICtrlSetState($radio2e, $GUI_ENABLE)
				If GUICtrlRead($radio2a) == $GUI_UNCHECKED Then
					GUICtrlSetState($input2a, $GUI_ENABLE)
					GUICtrlSetState($input2b, $GUI_ENABLE)
					GUICtrlSetState($button2a, $GUI_ENABLE)
					GUICtrlSetState($button2b, $GUI_ENABLE)
					GUICtrlSetState($button2c, $GUI_ENABLE)
				EndIf
			EndIf
			$titleEnable = 0
		EndIf

		If $positionDisable Then
			GUICtrlSetState($radio3a, $GUI_DISABLE)
			GUICtrlSetState($radio3b, $GUI_DISABLE)
			GUICtrlSetState($radio3c, $GUI_DISABLE)
			GUICtrlSetState($input3a, $GUI_DISABLE)
			GUICtrlSetState($radio3d, $GUI_DISABLE)
			GUICtrlSetState($radio3e, $GUI_DISABLE)
			GUICtrlSetState($radio3f, $GUI_DISABLE)
			GUICtrlSetState($input3d, $GUI_DISABLE)
			GUICtrlSetState($radio3g, $GUI_DISABLE)
			GUICtrlSetState($radio3h, $GUI_DISABLE)
			GUICtrlSetState($radio3i, $GUI_DISABLE)
			GUICtrlSetState($input3g, $GUI_DISABLE)
			GUICtrlSetState($radio3j, $GUI_DISABLE)
			GUICtrlSetState($radio3k, $GUI_DISABLE)
			GUICtrlSetState($radio3l, $GUI_DISABLE)
			GUICtrlSetState($input3j, $GUI_DISABLE)
			GUICtrlSetState($button3a, $GUI_DISABLE)
			$positionDisable = 0
		EndIf

		If $positionEnable Then
			If GUICtrlRead($radio1a) == $GUI_CHECKED AND GUICtrlRead($radio2a) == $GUI_UNCHECKED Then
				GUICtrlSetState($radio3a, $GUI_ENABLE)
				GUICtrlSetState($radio3b, $GUI_ENABLE)
				GUICtrlSetState($radio3c, $GUI_ENABLE)
				If GUICtrlRead($radio3c) == $GUI_CHECKED Then
					GUICtrlSetState($input3a, $GUI_ENABLE)
				Else
					GUICtrlSetState($input3a, $GUI_DISABLE)
				EndIf
				GUICtrlSetState($radio3d, $GUI_ENABLE)
				GUICtrlSetState($radio3e, $GUI_ENABLE)
				GUICtrlSetState($radio3f, $GUI_ENABLE)
				If GUICtrlRead($radio3f) == $GUI_CHECKED Then
					GUICtrlSetState($input3d, $GUI_ENABLE)
				Else
					GUICtrlSetState($input3d, $GUI_DISABLE)
				EndIf
				GUICtrlSetState($radio3g, $GUI_ENABLE)
				GUICtrlSetState($radio3h, $GUI_ENABLE)
				GUICtrlSetState($radio3i, $GUI_ENABLE)
				If GUICtrlRead($radio3i) == $GUI_CHECKED Then
					GUICtrlSetState($input3g, $GUI_ENABLE)
				Else
					GUICtrlSetState($input3g, $GUI_DISABLE)
				EndIf
				GUICtrlSetState($radio3j, $GUI_ENABLE)
				GUICtrlSetState($radio3k, $GUI_ENABLE)
				GUICtrlSetState($radio3l, $GUI_ENABLE)
				If GUICtrlRead($radio3l) == $GUI_CHECKED Then
					GUICtrlSetState($input3j, $GUI_ENABLE)
				Else
					GUICtrlSetState($input3j, $GUI_DISABLE)
				EndIf
				GUICtrlSetState($button3a, $GUI_ENABLE)
			EndIf
			$positionEnable = 0
		EndIf

		If $keysDisable Then
			GUICtrlSetState($input4a, $GUI_DISABLE)
			GUICtrlSetState($button4a, $GUI_DISABLE)
			$keysDisable = 0
		EndIf

		If $keysEnable Then
			If GUICtrlRead($radio1a) == $GUI_CHECKED AND GUICtrlRead($radio2a) == $GUI_UNCHECKED Then
				GUICtrlSetState($input4a, $GUI_ENABLE)
				GUICtrlSetState($button4a, $GUI_ENABLE)
			EndIf
			$keysEnable = 0
		EndIf


		$msg = GUIGetMsg()

		Select
			Case $msg == $radio1a
				; Run normal
				$titleEnable = 1
				$positionEnable = 1
				$keysEnable = 1
			Case $msg == $radio1b
				; Run hidden
				$titleDisable = 1
				$positionDisable = 1
				$keysDisable = 1
			Case $msg == $radio1c
				; Run minimized
				$titleEnable = 1
				$positionDisable = 1
				$keysEnable = 1
			Case $msg == $radio1d
				; Run maximized
				$titleEnable = 1
				$positionDisable = 1
				$keysEnable = 1
			Case $msg == $radio2a
				; Title match disable
				$titleDisable = 1
				$positionDisable = 1
				$keysDisable = 1
			Case $msg == $radio2b
				; Title match exact
				$titleEnable = 1
				$positionEnable = 1
				$keysEnable = 1
			Case $msg == $radio2c
				; Title match start
				$titleEnable = 1
				$positionEnable = 1
				$keysEnable = 1
			Case $msg == $radio2d
				; Title match contains
				$titleEnable = 1
				$positionEnable = 1
				$keysEnable = 1
			Case $msg == $radio2e
				; Title match start/end
				$titleEnable = 1
				$positionEnable = 1
				$keysEnable = 1
			Case $msg == $radio3a
				; Default position
				GUICtrlSetState($input3a, $GUI_DISABLE)
			Case $msg == $radio3b
				; Center position
				GUICtrlSetState($input3a, $GUI_DISABLE)
			Case $msg == $radio3c
				; Absolute position
				GUICtrlSetState($input3a, $GUI_ENABLE)
			Case $msg == $radio3d
				; Default position
				GUICtrlSetState($input3d, $GUI_DISABLE)
			Case $msg == $radio3e
				; Center position
				GUICtrlSetState($input3d, $GUI_DISABLE)
			Case $msg == $radio3f
				; Absolute position
				GUICtrlSetState($input3d, $GUI_ENABLE)
			Case $msg == $radio3g
				; Default position
				GUICtrlSetState($input3g, $GUI_DISABLE)
			Case $msg == $radio3h
				; Center position
				GUICtrlSetState($input3g, $GUI_DISABLE)
			Case $msg == $radio3i
				; Absolute position
				GUICtrlSetState($input3g, $GUI_ENABLE)
			Case $msg == $radio3j
				; Default position
				GUICtrlSetState($input3j, $GUI_DISABLE)
			Case $msg == $radio3k
				; Center position
				GUICtrlSetState($input3j, $GUI_DISABLE)
			Case $msg == $radio3l
				; Absolute position
				GUICtrlSetState($input3j, $GUI_ENABLE)
			Case $msg == $radio5a
				; Wait closed
				GUICtrlSetState($input5a, $GUI_ENABLE)
				GUICtrlSetState($input5b, $GUI_DISABLE)
			Case $msg == $radio5b
				; Wait n seconds
				GUICtrlSetState($input5a, $GUI_DISABLE)
				GUICtrlSetState($input5b, $GUI_ENABLE)
			Case $msg == $button1a
				; Browse for Program
				GUISetState($GUI_DISABLE, $dlg)
				$item = FileOpenDialog($TitleBar & $TitlePBrowse, @ScriptDir, "Executables (*.exe)", 1)
				If StringLen($item) Then
					If $flag Then $item = ConvertRelativePath($item, @ScriptDir)
					GUICtrlSetData($input1a, $item)
					$item2 = GUICtrlRead($input1b)
					If StringLen($item2) == 0 Then
						$item2 = StringLeft($item, StringInStr($item, "\", 0, -1) - 1)
						GUICtrlSetData($input1b, $item2)
					EndIf
				EndIf
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button1b
				; Browse for Starting Directory
				GUISetState($GUI_DISABLE, $dlg)
				$item = FileSelectFolder($TitleBar & $TitleDBrowse, "", 0, @ScriptDir)
				If StringLen($item) AND $flag Then
					$item = ConvertRelativePath($item, @ScriptDir)
					GUICtrlSetData($input1b, $item)
				EndIf
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button1c
				; Convert params path info
				$item = GUICtrlRead($input1c)
				If StringLen($item) AND $flag Then
					$item2 = GUICtrlRead($input1a)
					If StringInStr($item2, "\") Then
						$item2 = StringLeft($item2, StringInStr($item2, "\", 0, -1))
						If StringRight($item2, 1) == "\" Then
							$item2 = StringLeft($item2, StringLen($item2) - 1)
						EndIf
					EndIf
					$item = ConvertRelativePath($item, $item2)
					GUICtrlSetData($input1c, $item)
				EndIf
			Case $msg == $button2a
				; Browse for Starting Title
				GUISetState($GUI_DISABLE, $dlg)
				$item = BrowseForTitle()
				If StringLen($item) Then
					GUICtrlSetData($input2a, $item)
				EndIf
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button2b
				; Browse for Ending Title
				GUISetState($GUI_DISABLE, $dlg)
				$item = BrowseForTitle()
				If StringLen($item) Then
					GUICtrlSetData($input2b, $item)
				EndIf
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button1d
				; Test program button
				$item = GUICtrlRead($input1a)
				$item2 = GUICtrlRead($input1b)
				$item3 = GUICtrlRead($input1c)
				If GUICtrlRead($check1) == $GUI_CHECKED Then
					$item4 = 1
				Else
					$item4 = 0
				EndIf
				If GUICtrlRead($radio1a) == $GUI_CHECKED Then
					$item5 = ""
				ElseIf GUICtrlRead($radio1b) == $GUI_CHECKED Then
					$item5 = @SW_HIDE
				ElseIf GUICtrlRead($radio1c) == $GUI_CHECKED Then
					$item5 = @SW_MINIMIZE
				Else
					$item5 = @SW_MAXIMIZE
				EndIf
				GUISetState($GUI_DISABLE, $dlg)
				TestProgram($item, $item2, $item3, $item4, $item5)
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button2c
				; Test title match button
				$item = GUICtrlRead($input1a)
				If GUICtrlRead($radio2a) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 1)
					$item2 = $IniRunTitleModeNone
				ElseIf GUICtrlRead($radio2b) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 3)
					$item2 = $IniRunTitleModeExact
				ElseIf GUICtrlRead($radio2c) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 1)
					$item2 = $IniRunTitleModeStart
				ElseIf GUICtrlRead($radio2d) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 2)
					$item2 = $IniRunTitleModeContains
				Else
					AutoItSetOption("WinTitleMatchMode", 4)
					$item2 = $IniRunTitleModeRegexp
				EndIf
				$item3 = GUICtrlRead($input2a)
				$item4 = GUICtrlRead($input2b)
				GUISetState($GUI_DISABLE, $dlg)
				TestTitle($item, $item2, $item3, $item4)
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button3a
				; Test window position button
				If GUICtrlRead($radio2a) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 1)
					$item = $IniRunTitleModeNone
				ElseIf GUICtrlRead($radio2b) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 3)
					$item = $IniRunTitleModeExact
				ElseIf GUICtrlRead($radio2c) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 1)
					$item = $IniRunTitleModeStart
				ElseIf GUICtrlRead($radio2d) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 2)
					$item = $IniRunTitleModeContains
				Else
					AutoItSetOption("WinTitleMatchMode", 4)
					$item = $IniRunTitleModeRegexp
				EndIf
				$item2 = GUICtrlRead($input2a)
				$item3 = GUICtrlRead($input2b)
				GUISetState($GUI_DISABLE, $dlg)
				$item = GetWindowSearchText($item, $item2, $item3)
				If $item <> "" Then
					If GUICtrlRead($radio3a) == $GUI_CHECKED Then
						$item2 = $IniRunDefault
					ElseIf GUICtrlRead($radio3b) == $GUI_CHECKED Then
						$item2 = $IniRunCenter
					Else
						$item2 = GUICtrlRead($input3a)
					EndIf

					If GUICtrlRead($radio3d) == $GUI_CHECKED Then
						$item3 = $IniRunDefault
					ElseIf GUICtrlRead($radio3e) == $GUI_CHECKED Then
						$item3 = $IniRunCenter
					Else
						$item3 = GUICtrlRead($input3d)
					EndIf

					If GUICtrlRead($radio3g) == $GUI_CHECKED Then
						$item4 = $IniRunDefault
					ElseIf GUICtrlRead($radio3h) == $GUI_CHECKED Then
						$item4 = $IniRunCenter
					Else
						$item4 = GUICtrlRead($input3g)
					EndIf

					If GUICtrlRead($radio3j) == $GUI_CHECKED Then
						$item5 = $IniRunDefault
					ElseIf GUICtrlRead($radio3k) == $GUI_CHECKED Then
						$item5 = $IniRunCenter
					Else
						$item5 = GUICtrlRead($input3j)
					EndIf
					TestPosition($item, $item4, $item5, $item2, $item3)
				EndIf
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button4a
				; Test send keys button
				If GUICtrlRead($radio2a) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 1)
					$item = $IniRunTitleModeNone
				ElseIf GUICtrlRead($radio2b) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 3)
					$item = $IniRunTitleModeExact
				ElseIf GUICtrlRead($radio2c) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 1)
					$item = $IniRunTitleModeStart
				ElseIf GUICtrlRead($radio2d) == $GUI_CHECKED Then
					AutoItSetOption("WinTitleMatchMode", 2)
					$item = $IniRunTitleModeContains
				Else
					AutoItSetOption("WinTitleMatchMode", 4)
					$item = $IniRunTitleModeRegexp
				EndIf
				$item2 = GUICtrlRead($input2a)
				$item3 = GUICtrlRead($input2b)
				GUISetState($GUI_DISABLE, $dlg)
				$item = GetWindowSearchText($item, $item2, $item3)
				If $item <> "" Then
					$item2 = GUICtrlRead($input4a)
					TestKeys($item, $item2)
				EndIf
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button5a
				; Test program close button
				$item = GUICtrlRead($input1a)
				If GUICtrlRead($Radio5a) == $GUI_CHECKED Then
					$item2 = $IniRunWaitClose
					$item3 = GUICtrlRead($input5a)
				Else
					$item2 = GUICtrlRead($input5b)
					$item3 = 0
				EndIf
				GUISetState($GUI_DISABLE, $dlg)
				TestWait($item, $item2, $item3)
				GUISetState($GUI_ENABLE, $dlg)
				GUISetState(@SW_HIDE, $dlg)
				GUISetState(@SW_SHOW, $dlg)
			Case $msg == $button1
				; Done button
				$item = GUICtrlRead($input1a)
				If StringLen($item) Then
					$cancelled = 0
					$notDone = 0
				Else
					MsgBox(0, $TitleBar, "Error - You must provide a program executable.")
					GUICtrlSetState($input1a, $GUI_FOCUS)
				EndIf
			Case $msg == $button2
				; Cancel button
				$cancelled = 1
				$notDone = 0
		EndSelect
	WEnd


	; 4. Update profile name if needed.
	If $cancelled == 0 Then
		; Write program info
		$item = GUICtrlRead($input1a)
		$item2 = GUICtrlRead($input1b)
		$item3 = GUICtrlRead($input1c)
		If GUICtrlRead($check1) == $GUI_CHECKED Then
			$item4 = 1
		Else
			$item4 = 0
		EndIf
		If GUICtrlRead($radio1a) == $GUI_CHECKED Then
			$item5 = $IniRunProgramModeNormal
		ElseIf GUICtrlRead($radio1b) == $GUI_CHECKED Then
			$item5 = $IniRunProgramModeHide
		ElseIf GUICtrlRead($radio1c) == $GUI_CHECKED Then
			$item5 = $IniRunProgramModeMin
		Else
			$item5 = $IniRunProgramModeMax
		EndIf
		WriteProgramInfoToIni($section, $item, $item2, $item3, $item4, $item5)

		; Write title match info
		If GUICtrlRead($radio2a) == $GUI_CHECKED Then
			$item = $IniRunTitleModeNone
		ElseIf GUICtrlRead($radio2b) == $GUI_CHECKED Then
			$item = $IniRunTitleModeExact
		ElseIf GUICtrlRead($radio2c) == $GUI_CHECKED Then
			$item = $IniRunTitleModeStart
		ElseIf GUICtrlRead($radio2d) == $GUI_CHECKED Then
			$item = $IniRunTitleModeContains
		Else
			$item = $IniRunTitleModeRegexp
		EndIf
		$item2 = GUICtrlRead($input2a)
		$item3 = GUICtrlRead($input2b)
		WriteTitleInfoToIni($section, $item, $item2, $item3)
	
		; Write send keystroke info
		$item = GUICtrlRead($input4a)
		WriteKeysInfoToIni($section, $item)

		; Write delay info.
		If GUICtrlRead($Radio5a) == $GUI_CHECKED Then
			$item = $IniRunWaitClose
			$item2 = GUICtrlRead($input5a)
		Else
			$item = GUICtrlRead($input5b)
			$item2 = 0
		EndIf
		WriteDelayInfoToIni($section, $item, $item2)

		; Write window positioning info
		If GUICtrlRead($radio3a) == $GUI_CHECKED Then
			$item = $IniRunDefault
		ElseIf GUICtrlRead($radio3b) == $GUI_CHECKED Then
			$item = $IniRunCenter
		Else
			$item = GUICtrlRead($input3a)
		EndIf

		If GUICtrlRead($radio3d) == $GUI_CHECKED Then
			$item2 = $IniRunDefault
		ElseIf GUICtrlRead($radio3e) == $GUI_CHECKED Then
			$item2 = $IniRunCenter
		Else
			$item2 = GUICtrlRead($input3d)
		EndIf

		If GUICtrlRead($radio3g) == $GUI_CHECKED Then
			$item3 = $IniRunDefault
		ElseIf GUICtrlRead($radio3h) == $GUI_CHECKED Then
			$item3 = $IniRunCenter
		Else
			$item3 = GUICtrlRead($input3g)
		EndIf

		If GUICtrlRead($radio3j) == $GUI_CHECKED Then
			$item4 = $IniRunDefault
		ElseIf GUICtrlRead($radio3k) == $GUI_CHECKED Then
			$item4 = $IniRunCenter
		Else
			$item4 = GUICtrlRead($input3j)
		EndIf
		WritePositionInfoToIni($section, $item3, $item4, $item, $item2)
	EndIf


	; 5. Return new profile name.
	GUIDelete($dlg)
	Return(BitXOR($cancelled, 1))
EndFunc

;;
; User Defined Functions
; Dialog Support Functions
;

;
; BuildProfilesList($control, $selection)
;
; Entry:		$control	GUI control to fill
;			    $selection	Selection to make current
; Return:		$Items		Size of list
; Procedure:	1.  Get item count.
;			    2. Read and add aitems.
;			    3. Set list and current.
;			    4. Return count.
;
Func	BuildProfilesList($control, $selection)
	Local	$list, $entry
	Local	$items, $index, $count, $item

	; 1. Get item count.
	$list = ""
	$count = 0
	$entry = $selection
	$items = Number(IniRead($IniWorkFile, $IniProfilesSection, $IniItems, 0))


	; 2. Read and add aitems.
	$index = 1
	While $index <= $items
		$item = IniRead($IniWorkFile, $IniProfilesSection, $IniItem & $index, "")
		If StringLen($item) Then
			$list &= $SepChar & $item
			$count += 1
			If StringLen($entry) == 0 Then $entry = $item
		EndIf
		$index += 1
	WEnd


	; 3. Set list and current.
	GUICtrlSetData($control, $list, $entry)

	
	; 4. Return count.
	Return($count)
EndFunc


;
; BuildProgramst($control, $profile, $select)
;
; Entry:		$control	GUI control to fill
;			    $profile	Profile to build list for
;			    $select		Item to set as current
; Return:		$Items		Size of list
; Procedure:	1. Get item count.
;			    2. Read and add aitems.
;			    3. Set list and current.
;			    4. Return count.
;
Func	BuildProgramsList($control, $profile, $select)
	Local	$entry
	Local	$items, $index, $count
	Local	$item, $item2

	; 1. Get item count.
	$entry = ""
	$count = 0
	$list = ""
	$items = Number(IniRead($IniWorkFile, $profile, $IniItems, 0))


	; 2. Read and add aitems.
	$index = 1
	While $index <= $items
		$item = $profile & "-" & StringRight("00" & $index, 2)
		$item2 = IniRead($IniWorkFile, $item, $IniRunProgram, "")
		If StringInStr($item2, "\") Then
			$item2 = StringMid($item2, StringInStr($item2, "\", 0, -1) + 1)
		EndIf

		If StringLen($item2) Then
			$list &= $SepChar & $item2
			If $select == $count Then $entry = $item2
			$count += 1
		EndIf
		$index += 1
	WEnd



	; 3. Set list and current.
	GUICtrlSetData($control, $list, $entry)

	
	; 4. Return count.
	Return($count)
EndFunc


;
; BrowseForTitle()
;
; Entry:		<None>
; Return:		$title		Title selected by user
;			    NULL		If user aborts
; Procedure:	1. Gather titles.
;			    2. Build dialog.
;			    3. Get user selection.
;			    4. Return data.
;
Func	BrowseForTitle()
	Local	$title, $index, $list, $notDone
	Local	$dlg
	Local	$list1
	Local	$button1, $button2

	; 1. Gather the titles.
	$titles = WinList()
	If $titles[0][0] == 0 Then Return("")


	; 2. Build the dialog.
	$dlg = GUICreate($TitleBar & $TitleTBrowse, 280, 340)
	GUICtrlCreateLabel($TextTitles, 20, 10, 210, 20)
	$list1 = GUICtrlCreateList("", 20, 30, 210, 140, $WS_VSCROLL)
	$button1 = GUICtrlCreateButton($TextOk, 40, 290, 80, 30)
	$button2 = GUICtrlCreateButton($TextCancel, 140, 290, 80, 30)

	
	; 2. Populate dialog
	$list = ""
	$index = 1
	While $index <= $titles[0][0]
		If StringLen($titles[$index][0]) AND BitAnd(WinGetState($titles[$index][1]), 2) Then
			$list &= $SepChar & $titles[$index][0]
		EndIf
		$index += 1
	WEnd
	GUICtrlSetData($list1, $list)
	GUISetState(@SW_SHOW, $dlg)


	; 3. Get user selection.
	$notDone = 1
	While $notDone
		$msg = GUIGetMsg()

		Select
			Case $msg == $button1
				; OK button
				$title = GUICtrlRead($list1)
				If StringLen($title) Then
					$notDone = 0
				Else
					MsgBox(0, $TitleBar2, "Error - No title selected")
				EndIf
			Case $msg == $button2
				$title = ""
				$notDone = 0
				; Cancel button
		EndSelect
	WEnd


	; 4. Return data
	GUIDelete($dlg)
	Return($title)
EndFunc


;
; ConvertRelativePath($text, $path)
;
; Entry:		$text		String to convert
;			    $path		Path to be relative from
; Return:		String with paths converted to relative
; Procedure:	1. Fixup parent path.
;			    2. Convert the data.
;			    3. Return results.
;
Func	ConvertRelativePath($text, $path)
	Local	$path2, $path3, $path4
	Local	$text2, $text3, $text4
	Local	$notDone

	; 1. Fixup parent path.
	$path2 = ""
	$path3 = $path

	If StringLeft($path3, 1) == "\" Then
		$path3 = StringLeft(@ScriptDir, 2) & $path3
	EndIf

	While StringLeft($path3, 3) == "..\"
		If StringLen($path2) == 0 Then $path2 = @ScriptDir
		If StringInStr($path2, "\") Then
			$path2 = StringLeft($path2, StringInStr($path2, "\", 0, -1) - 1)
			$path3 = StringMid($path3, 4)
		EndIf
	WEnd

	If StringMid($path3, 2, 2) <> ":\" Then
		If StringLen($path2) == 0 Then $path2 = @ScriptDir
		If StringRight($path2, 1) <> "\" Then $path2 &= "\"
		$path2 &= $path3
	Else
		$path2 = $path3
	EndIf


	; 2. Convert the data.
	$text2 = ""
	$text3 = $text
	While StringLen($text3)
		While StringLeft($text3, 1) == " " OR StringLeft($text3, 1) == @TAB OR StringLeft($text3, 1) == Chr(34)
			$text2 &= StringLeft($text3, 1)
			$text3 = StringMid($text3, 2)
		WEnd
		
		$text4 = ""
		While StringLen($text3) AND StringLeft($text3, 1) <> " " AND StringLeft($text3, 1) <> @TAB AND StringLeft($text3, 1) <> Chr(34)
			$text4 &= StringLeft($text3, 1)
			$text3 = StringMid($text3, 2)
		WEnd

		If StringLeft($text4, 3) == StringLeft($path2, 3) Then
			If StringLeft($text4, StringLen($path2)) == $path2 Then
				$text4 = StringMid($text4, StringLen($path2) + 1)
				If StringLeft($text4, 1) == "\" Then
					$text4 = StringMid($text4, 2)
				EndIf
			Else
				$path3 = ""
				$path4 = $path2
				$notDone = 1
				While $notDone
					If $path4 == "" Then
						$notDone = 0
					ElseIf StringLeft($text4, StringLen($path4)) == $path4 Then
						$text4 = StringMid($text4, StringLen($path4) + 1)
						If StringLeft($text4, 1) == "\" Then
							$text4 = StringMid($text4, 2)
						EndIf
						$text4 = $path3 & $text4
						$notDone = 0
					Else
						$path3 &= "..\"
						$path4 = StringLeft($path4, StringInStr($path4, "\", 0, -1) - 1)
					EndIf
				WEnd
			EndIf
		EndIf
		$text2 &= $text4
	WEnd


	; 3. Return results
	Return($text2)
EndFunc


;
; TestProgram($prog, $dir, $params, $single, $mode)
;
; Entry:		$prog		Program to run
;			    $dir		Starting directory.
;			    $params		Program parameters
;			    $single		Single instance flag
;			    $mode		Window display mode
; Return:		<None>
; Procedure:	1. Error if no program specified.
;			    2. Run the program.
;			    3. Report results.
;			    4. Exit all done.
;
Func	TestProgram($prog, $dir, $params, $single, $mode)
	; 1. Error if no program specified.
	If $prog == "" Then
		MsgBox(0, $TitleBar2, "Error - No program specified.")
		Return(0)
	EndIf


	; 2. Run the program.
	LaunchProgram($prog, $dir, $params, $single, $mode)
	Sleep(3000)

	; 3. Report results.
	Select
		Case @error == 0
			MsgBox(0, $TitleBar2, "Program launched successfully.")
		Case @error == -1
			MsgBox(0, $TitleBar2, "Program was already running (single instance)")
		Case @error == 1
			MsgBox(0, $TitleBar2, "Error - Program not found.")
		Case @error == 2
			MsgBox(0, $TitleBar2, "Error - Starting directory does not exist.")
		Case @error == 3
			MsgBox(0, $TitleBar2, "Error - Starting directory is not a directory.")
		Case @error == 4
			MsgBox(0, $TitleBar2, "Error - Program failed to launch.")
	EndSelect


	; 4. Exit all done.
	Return(0)
EndFunc


;
; TestTitle($prog, $mode, $title1, $title2)
;
; Entry:		$prog		Program to test for
;			    $mode		Window title match mode
;			    $title1		Title start text
;			    $title2		Title end text
; Return:		<None>
; Procedure:	1. Error if no program specified.
;			    2. Test for title.
;			    3. Report results.
;			    4. Exit all done.
;
Func	TestTitle($prog, $mode, $title1, $title2)
	; 1. Error if no program specified.
	If $prog == "" Then
		MsgBox(0, $TitleBar2, "Error - No program specified.")
		Return(0)
	EndIf


	; 2. Test for title.
	WaitProgramWindow($prog, $mode, $title1, $title2)

	; 3. Report results.
	Select
		Case @error == 0
			MsgBox(0, $TitleBar2, "Program window found successfully.")
		Case @error == 1
			MsgBox(0, $TitleBar2, "Error - Program not running.")
		Case @error == 2
			MsgBox(0, $TitleBar2, "Error - Invalid title text.")
		Case @error == 3
			MsgBox(0, $TitleBar2, "Error - Program window not found.")
	EndSelect


	; 4. Exit all done.
	Return(0)
EndFunc


;
; TestPosition($seachText, $width, $height, $xpos, $ypos)
;
; Entry:		$searchText	Title text
;			    $width		Window width
;			    $height		Window height
;			    $xpos		Window X position
;			    $ypos		Window Y position
; Return:		<None>
; Procedure:	1. Position the window.
;			    2. Report results.
;			    3. Exit all done.
;
Func	TestPosition($searchText, $width, $height, $xpos, $ypos)

	; 1. Position the window.
	PositionProgramWindow($searchText, $width, $height, $xpos, $ypos)


	; 2. Report results.
	If @error Then
		MsgBox(0, $TitleBar2, "Error - Program window not found.")
	Else
		MsgBox(0, $TitleBar2, "Window positioned successfully.")
	EndIf


	; 3. Exit all done.
	Return(0)
EndFunc


;
; TestKeys($seachText, $keys)
;
; Entry:		$searchText	Title text
;			    $keys		Keystrokes to send
; Return:		<None>
; Procedure:	1. Error if null data.
;			    2. Send the keystrokes.
;			    3. Report results.
;			    4. Exit all done.
;
Func	TestKeys($searchText, $keys)

	; 1. Error if null data.
	If $keys == "" Then
		MsgBox(0, $TitleBar2, "Error - No keystrokes to send.")
		Return(0)
	EndIf


	; 2. Send the keystrokes.
	SendProgramKeys($searchText, $keys)


	; 2. Report results.
	If @error Then
		MsgBox(0, $TitleBar2, "Error - Program window not found.")
	Else
		MsgBox(0, $TitleBar2, "Keystrokes successfully sent to program window.")
	EndIf


	; 3. Exit all done.
	Return(0)
EndFunc


;
; TestWait($program, $delay, $timeout)
;
; Entry:		$program	Program to wait on
;			    $delay		Time to wait
;			    $timeout	Timeout if wait close
; Return:		<None>
; Procedure:	1. Issue the wait.
;			    2. Report results.
;			    3. Exit all done.
;
Func	TestWait($program, $delay, $timeout)

	; 1. Issue the wait.
	WaitProgram($program, $delay, $timeout)


	; 2. Report results.
	If @error Then
		MsgBox(0, $TitleBar2, "Error - Timeout waiting fro program to close.")
	Else
		MsgBox(0, $TitleBar2, "Wait succeessful.")
	EndIf


	; 3. Exit all done.
	Return(0)
EndFunc


;
; GetWindowSearchText($mode, $title1, $title2)
;
; Entry:	   $mode		Window title match mode
;			    $title1		Title start text
;			    $title2		Title end text
; Return:		$searchText	Search string
; Procedure:	1. Error if title match mode disabled.
;			    2. Convert the title.
;			    3. Exit all done.
;
Func	GetWindowSearchText($mode, $title1, $title2)
	Local	$searchText

	; 1. Error if title match mode disabled.
	If $mode == $IniRunTitleModeNone Then
		MsgBox(0, $TitleBar2, "Error - Title match mode is disabbled.")
		Return("")
	EndIf


	; 2. Convert the title.
	If $mode == $IniRunTitleModeRegexp Then
		AutoItSetOption("WinTitleMatchMode", 4)
		$searchText = $RegexpHeader
		If StringLen($title1) AND StringLen($title2) Then
			$searchText &= $RegexpLeft1
			$searchText &= RegexpFix($title1)
			$searchText &= $RegexpLeft2
			$searchText &= $RegexpMid
			$searchText &= $RegexpRight1
			$searchText &= RegexpFix($title2)
			$searchText &= $regexpRight2
		ElseIf StringLen($title1) Then
			$searchText &= $RegexpLeft1
			$searchText &= RegexpFix($title1)
			$searchText &= $RegexpLeft2
			$searchText &= $RegexpMid
		ElseIf StringLen($title2) Then
			$searchText &= $RegexpMid
			$searchText &= $RegexpRight1
			$searchText &= RegexpFix($title2)
			$searchText &= $regexpRight2
		Else
			MsgBox(0, $TitleBar2, "Error - Invalid window title text.")
			Return("")
		EndIf
	Else
		$searchText = $Title1
		If StringLen($searchText) == 0 Then
			MsgBox(0, $TitleBar2, "Error - Invalid window title text.")
			Return("")
		EndIf
		If $mode == $IniRunTitleModeExact Then
			AutoItSetOption("WinTitleMatchMode", 3)
		ElseIf $mode == $IniRunTitleModeStart Then
			AutoItSetOption("WinTitleMatchMode", 1)
		Else
			AutoItSetOption("WinTitleMatchMode", 2)
		EndIf
	EndIf


	; 3. Exit all done.
	Return($searchText)
EndFunc


;
; User Defined Functions
; Ini/Mrn File Functions
;

;
; AddProfileToIni()
;
; Entry:		<None>
; Return:		New profile name
;			    @error = 1 if user aborted.
;			    @error = 2 if error updating profiles
;			    @error = 3 if error creating profile
; Procedure:	1. Get user to enter new profile name.
;			    2. Error if duplicate.
;			    3. Append to end.
;			    4. Update count.
;			    5. Create empty profile section.
;			    6. Exit all done.
;
Func	AddProfileToIni()
	Local	$items, $index, $item, $item2, $ret

	; 1. Get user to enter new profile name.
	$ret = 0
	$items = Number(IniRead($IniWorkFile, $IniProfilesSection, $IniItems, 0))
	While 1
		$item = InputBox($TitleBar2, "Please enter new profile name:", "", "", 300, 120)
		If @error Then
			SetError(1)
			Return("")
		EndIf
		
		If StringInStr($item, "[") OR StringInStr($item, "]") Then
			MsgBox(0, $TitleBar2, "Error - Profile name may not contain '[' or ']' characters.")
		Else
			; 2. Error if duplicate.
			$index = 1
			While $index <= $items
				$item2 = IniRead($IniWorkFile, $IniProfilesSection, $IniItem & $index, "")
				If $item == $Item2 Then ExitLoop
				$index += 1
			WEnd
		
			If $index <= $items Then
				MsgBox(0, $TitleBar2, "Error - '" & $item & "' already exists.")
			Else
				ExitLoop
			EndIf
		EndIf
	WEnd


	; 3. Append to end.
	If IniWrite($IniWorkFile, $IniProfilesSection, $IniItem & $index, $item) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure adding new profile name.")
		$ret = 2
	EndIf


	; 4. Update count.
	$items += 1
	If IniWrite($IniWorkFile, $IniProfilesSection, $IniItems, $items) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure updating profile count.")
		$ret = 2
	EndIf


	; 5. Create empty profile section.
	If IniWrite($IniWorkFile, $item, $IniSectionID, $IniProfileEntryID) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure writing new profile section ID.")
		$ret = 3
	EndIf

	If IniWrite($IniWorkFile, $item, $IniItems, 0) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure writing new profile count.")
		$ret = 3
	EndIf


	; 6. Exit all done.
	If $ret Then
		SetError($ret)
	EndIf
	
	Return($item)
EndFunc


;
; RenameProfile($old, $new)
;
; Entry:		$old			Name of profile to rename
;			$new			New profile name
; Return:		1 if all ok
;			    0 if error
;			    @error = 1 if old profile not found
;			    @error = 2 if invalid profile section
;			    @error = 3 if error updating profiles
;			    @error = 4 if error renaming profile
;			    @error = 5 if error renaming program entry
; Procedure:	1. Find item number.
;			    2. Get program count.
;			    3. Rename program sections.
;			    4. Rename profile section.
;			    5. Rename entry in profiles section
;			    6. Exit all done.
;
Func	RenameProfileInIni($old, $new)
	Local	$items, $index, $index2, $item, $item2, $ret

	; 1. Find item number.
	$ret = 0
	$items = Number(IniRead($IniWorkFile, $IniProfilesSection, $IniItems, 0))
	$index = 1
	While $index <= $items
		$item = IniRead($IniWorkFile, $IniProfilesSection, $IniItem & $index, "")
		If $item == $old Then ExitLoop
		$index += 1
	WEnd
	
	If $index > $items Then
		MsgBox(0, $TitleBar2, "Error - Profile '" & $old & "' not found.")
		SetError(1)
		Return(0)
	EndIf


	; 2. Get program count.
	$item = Number(IniRead($IniWorkFile, $old, $IniSectionID, 0))
	If $item <> $IniProfileEntryID Then
		MsgBox(0, $TitleBar2, "Error - Profile '" & $old & "' not valid.")
		SetError(2)
		Return(0)
	EndIF

	$items = Number(IniRead($IniWorkFile, $old, $IniItems, 0))
	If $old == $new Then Return(1)


	; 3. Rename program sections.
	$index2 = 1
	While $index2 <= $items
		$item = $old & StringRight("00" & $index2, 2)
		$item2 = $new & StringRight("00" & $index2, 2)
		
		If IniRenameSection($IniWorkFile, $item, $item2) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure renaming section '" & $item & "' to '" & $item2 & "'.")
			$ret = 5
		EndIf
		
		$index2 += 1
	WEnd


	; 4. Rename profile section.
	If IniRenameSection($IniWorkFile, $old, $new) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure renaming section '" & $old & "' to '" & $new & "'.")
		$ret = 4
	EndIf


	; 5. Rename entry in profiles section
	If IniWrite($IniWorkFile, $IniProfilesSection, $IniItem & $index, $new) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure rename profile entry '" & $old & "' to '" & $new & "'.")
		$ret = 3
	EndIf


	; 6. Exit all done.
	If $ret Then
		SetError($ret)
		Return(0)
	EndIf

	Return(1)
EndFunc


;
; DeleteProfile($profile)
;
; Entry:		$profile		Name of profile to delete
; Return:		1 if all ok
;			    0 if error
;			    @error = 1 if profile not found
;			    @error = 2 if error updating profiles
;			    @error = 3 if error deleting profile
;			    @error = 4 if error deleting program entry
; Procedure:	1. Find item number.
;			    2. Move remainder up.
;			    3. Delete last.
;			    4. Update count.
;			    5. Get program count.
;			    6. Delete program sections.
;			    7. Delete profile section.
;			    8. Exit all done.
;
Func	DeleteProfileFromIni($profile)
	Local	$items, $index, $item, $ret

	; 1. Find item number.
	$ret = 0
	$items = Number(IniRead($IniWorkFile, $IniProfilesSection, $IniItems, 0))
	$index = 1
	While $index <= $items
		$item = IniRead($IniWorkFile, $IniProfilesSection, $IniItem & $index, "")
		If $item == $profile Then ExitLoop
		$index += 1
	WEnd
	
	If $index > $items Then
		MsgBox(0, $TitleBar2, "Error - Profile '" & $profile & "' not found.")
		SetError(1)
		Return(0)
	EndIf


	; 2. Move remainder up.
	While $index < $items
		$item = IniRead($IniWorkFile, $IniProfilesSection, $IniItem & ($index + 1), "")
		If IniWrite($IniWorkFile, $IniProfilesSection, $IniItem & $index, $item) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Write failure shifting profile names.")
			$ret = 2
		EndIf
		$index += 1
	WEnd


	; 3. Delete last.
	If IniDelete($IniWorkFile, $IniProfilesSection, $IniItem & $index) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure deleting profile names.")
		$ret = 2
	EndIf


	; 4. Update count.
	$items -= 1
	If IniWrite($IniWorkFile, $IniProfilesSection, $IniItems, $items) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure updating profile count.")
		$ret = 2
	EndIf


	; 5. Get program count.
	$items = Number(IniRead($IniWorkFile, $profile, $IniItems, 0))


	; 6. Delete program sections.
	$index = 1
	While $index <= $items
		$item = StringRight("00" & $index, 2)
		If IniDelete($IniWorkFile, $profile & "-" & $item) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Write failure deleting program entry " & $item & ".")
			$ret = 4
		EndIf
		$index += 1
	WEnd


	; 7. Delete profile section.
	If IniDelete($IniWorkFile, $profile) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Write failure deleting profile section.")
		$ret = 3
	EndIf


	; 8. Exit all done.
	If $ret Then
		SetError($ret)
		Return(0)
	EndIf
	Return(1)
EndFunc


;
; Add ProgramEntryToIni($profile)
;
; Entry:		$profile		Profile to add entry to
; Return:		0 if fail
;			    Else program entry number
;			    @error = 1 if profile not found.
;			    @error = 2 if failure updating program count
;			    @error = 3 if failure writing new program entry
; Procediure:	1. Verify profile.
;			    2. Get program entry count.
;			    3. Create new program entry.
;			    4. Update program count.
;			    5. Exit all done.
;
Func	AddProgramEntryToIni($profile)
	Local	$ret, $items, $item, $section

	; 1. Verify profile.
	$item = Number(IniRead($IniWorkFile, $profile, $IniSectionID, 0))
	If $item <> $IniProfileEntryID Then
		MsgBox(0, $TitleBar2, "Error - Invalid section ID in profile '" & $profile & "'.")
		If IniWrite($IniWorkFile, $profile, $IniSectionID, $IniProfileEntryID) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure repairing section ID in profile '" & $profile & "'.")
		EndIf
		$ret = 1
	EndIf


	; 2. Get program entry count.
	$items = Number(IniRead($IniWorkFile, $profile, $IniItems, 0))
	$items += 1


	; 3. Create new program entry.
	$section = $profile & "-" & StringRight("00" & $items, 2)
	If IniWrite($IniWorkFile, $section, $IniSectionID, $IniRunSectionID) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure creating program entry section '" & $section & "'.")
		$ret = 3
	EndIf


	; 4. Update program count.
	If IniWrite($IniWorkFile, $profile, $IniItems, $items) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure updfating program count fro profile '" & $profile & "'.")
		$ret = 2
	EndIf


	; 5. Exit all done.
	If $ret Then
		SetError($ret)
	EndIf
	
	Return($items)
EndFunc


;
; SwapProgramEntriesInIni($profile, $index1, $index2)
;
; Entry:		$profile	Name of profile to modify
;			    $index1		Index of first entry
;			    $index2		Index of second emtry
; Return:		1 if all ok
;			    0 if error
;			    @error = 1 if profile not found
;			    @error = 2 if index1 is out of bounds
;			    @error = 3 if index2 is out of bounds
;			    @error = 4 if error renaming program entry
; Procedure:	1. Get count for profile, error if not found..
;			    2. Fail if indices are out of bounds.
;			    3. Build and check program entry sections.
;			    4. Rename index1 to indexX.
;			    5. Rename index2 to index1.
;			    6. Rename indexX to index2
;			    7. Exit all done.
;
Func	SwapProgramEntriesInIni($profile, $index1, $index2)
	Local	$items, $ret
	Local	$section1, $section2, $sectionX

	; 1. Get count for profile, error if not found..
	$ret = 0
	$items = Number(IniRead($IniWorkFile, $profile, $IniSectionID, 0))
	If $items <> $IniProfileEntryID Then
		MsgBox(0, $TitleBar2, "Error - Invalid or missing profile entry.")
		SetError(1)
		Return(0)
	EndIf
	$items = Number(IniRead($IniWorkFile, $profile, $IniItems, 0))


	; 2. Fail if indices are out of bounds.
	If $index1 == $index2 Then Return(1)
	If $index1 < 1 OR $index1 > $items Then
		SetError(2)
		Return(0)
	EndIf

	If $index2 < 1 OR $index2 > $items Then
		SetError(3)
		Return(0)
	EndIf


	; 3. Build and check program entry sections.
	$section1 = $profile & "-" & StringRight("00" & $index1, 2)
	$section2 = $profile & "-" & StringRight("00" & $index2, 2)
	$sectionX = $profile & "-" & "XX"


	; 4. Rename index1 to indexX.
	If IniRenameSection($IniWorkFile, $section1, $sectionX) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure renaming program section " & $index1 & ".")
		$ret = 4
	EndIf


	; 5. Rename index2 to index1.
	If IniRenameSection($IniWorkFile, $section2, $section1) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure renaming program section " & $index2 & ".")
		$ret = 4
	EndIf


	; 6. Rename indexX to index2
	If IniRenameSection($IniWorkFile, $sectionX, $section2) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure renaming program section X.")
		$ret = 4
	EndIf


	; 7. Exit all done.
	If $ret Then
		SetError($ret)
		Return(0)
	EndIf
	
	Return(1)
EndFunc


;
; DeleteProgramEntryFromIni($profile, $index)
;
; Entry:		$profile	Name of profile to delete
;			    $index		Index to delete
; Return:		1 if all ok
;			    0 if error
;			    @error = 1 if profile not found
;			    @error = 2 if error updating profiles
;			    @error = 3 if error deleting program entry
;			    @error = 4 if error renaming program entries
; Procedure:	1. Verify profile & get count.
;			    2. Delete program entry.
;			    3. Move remainder up.
;			    4. Update count.
;			    5. Exit all done.
;
Func	DeleteProgramEntryFromIni($profile, $index)
	Local	$items, $item, $ret
	Local	$section1, $section2

	; 1. Verify profile & get count.
	$item = Number(IniRead($IniWorkFile, $profile, $IniSectionID, 0))
	If $item <> $IniProfileEntryID Then
		MsgBox(0, $TitleBar2, "Error - Invalid section ID in profile '" & $profile & "'.")
		If IniWrite($IniWorkFile, $profile, $IniSectionID, $IniProfileEntryID) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure repairing section ID in profile '" & $profile & "'.")
		EndIf
		$ret = 1
	EndIf

	$items = Number(IniRead($IniWorkFile, $profile, $IniItems, 0))


	; 2. Delete program entry.
	$section1 = $profile & "-" & StringRight("00" & $index, 2)
	If IniDelete($IniWorkFile, $section1) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure deleting program entry section '" & $section1 & "'.")
		$ret = 3
	EndIf


	; 3. Move remainder up.
	$item = $index
	While $item < $items
		$section1 = $profile & "-" & StringRight("00" & $item, 2)
		$section2 = $profile & "-" & StringRight("00" & ($item + 1), 2)
		If IniRenameSection($IniWorkFile, $section2, $section1) <> 1 Then
			MsgBox(0, $TitleBar2, "Error - Failure renaming program entry section '" & $section2 & "'.")
			$ret = 4
		EndIf
		$item += 1
	WEnd


	; 4. Update program count.
	$items -= 1
	If IniWrite($IniWorkFile, $profile, $IniItems, $items) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure updfating program count fro profile '" & $profile & "'.")
		$ret = 2
	EndIf


	; 5. Exit all done.
	If $ret Then
		SetError($ret)
		Return(0)
	EndIf
	
	Return(1)
EndFunc


;
; WriteProgramInfoToIni($section, $program, $directory, $params, $single, $mode)
;
; Entry:		$section		Name of section to write to
;			    $program		Executable
;			    $directory		Starting directory
;			    $single		Single instance flag
;			    $mode		Run mode
; Return:		1 if all ok
;			    0 if write errors
; Procedure:	1. Write program executable.
;			    2. Write starting directory.
;			    3. Write cokmmand line parameters.
;			    4. Write program mode.
;			    5. Write single instance flag.
;			    6. Exit all done.
;
Func	WriteProgramInfoToIni($section, $program, $directory, $params, $single, $mode)
	Local	$ret, $item

	; 1. Write program executable.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunProgram, $program) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing program name to Ini file.")
		$ret = 0
	EndIf


	; 2. Write starting directory.
	If IniWrite($IniWorkFile, $section, $IniRunDir, $directory) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing starting directory name to Ini file.")
		$ret = 0
	EndIf


	; 3. Write command line parameters.
	If StringLeft($params, 1) == Chr(34) AND StringRight($params, 1) == Chr(34) Then
		$item = Chr(34) & $params & Chr(34)
	Else
		$item = $params)
	EndIf
	If IniWrite($IniWorkFile, $section, $IniRunParams, $item) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing program parameters to Ini file.")
		$ret = 0
	EndIf


	; 4. Write program mode.
	If IniWrite($IniWorkFile, $section, $IniRunProgramMode, $mode) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing program mode to Ini file.")
		$ret = 0
	EndIf


	; 5. Write single instance flag.
	If IniWrite($IniWorkFile, $section, $IniRunSingleInstance, $single) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing single instance flag to Ini file.")
		$ret = 0
	EndIf


	; 6. Exit all done.
	Return($ret)
EndFunc


;
; WriteTitleInfoToIni($section, $mode, $title1, $title2)
;
; Entry:		$section	Name of section to write to
;			    $mode		Title match mode
;			    $title1		Starting title text
;			    $title2		Ending title text
; Return:		1 if all ok
;			    0 if write errors
; Procedure:	1. Write title match mode.
;			    2. Write starting title text.
;			    3. Write ending title text.
;			    4. Exit all done.
;
Func	WriteTitleInfoToIni($section, $mode, $title1, $title2)
	Local	$ret

	; 1. Write title match mode.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunTitleMode, $mode) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing title match mode to Ini file.")
		$ret = 0
	EndIf


	; 2. Write starting title text.
	If IniWrite($IniWorkFile, $section, $IniRunTitle1, $title1) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing starting title text to Ini file.")
		$ret = 0
	EndIf


	; 3. Write ending title text.
	If IniWrite($IniWorkFile, $section, $IniRunTitle2, $title2) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing ending title text to Ini file.")
		$ret = 0
	EndIf


	; 4. Exit all done.
	Return($ret)
EndFunc


;
; WriteKeysInfoToIni($section, $keys)
;
; Entry:		$section	Name of section to write to
;			    $keys		Key sequence to write to window
; Return:		1 if all ok
;			    0 if write errors
; Procedure:	1. Write key sequence.
;			    2. Exit all done.
;
Func	WriteKeysInfoToIni($section, $keys)
	Local	$ret

	; 1. Write key sequence.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunKeys, $keys) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing key sequence to Ini file.")
		$ret = 0
	EndIf


	; 2. Exit all done.
	Return($ret)
EndFunc


;
; WriteDelayInfoToIni($section, $wait, $timeout)
;
; Entry:		$section	Name of section to write to
;			    $wait		Delay time period
;			    $timeout	Timeout if wait to close
; Return:		1 if all ok
;			    0 if write errors
; Procedure:	1. Write delay time.
;			    2. Write timeout.
;			    3. Exit all done.
;
Func	WriteDelayInfoToIni($section, $wait, $timeout)
	Local	$ret

	; 1. Write delay time..
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunWait, $wait) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing wait time to Ini file.")
		$ret = 0
	EndIf


	; 2. Write timeout.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunTimeout, $timeout) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing timeout to Ini file.")
		$ret = 0
	EndIf


	; 3. Exit all done.
	Return($ret)
EndFunc


;
; WritePositionInfoToIni($section, $width, $height, $xpos, $ypos)
;
; Entry:		$section	Name of section to write to
;			    $width		Width of window
;			    $height		Height of window
;			    $xpos		X position of window
;			    $ypos		Y position of window
; Return:		1 if all ok
;			    0 if write errors
; Procedure:	1. Write width.
;			    2. Write height.
;			    3. Write X position.
;			    4. Write Y position.
;			    5. Exit all done.
;
Func	WritePositionInfoToIni($section, $width, $height, $xpos, $ypos)
	Local	$ret

	; 1. Write width..
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunWidth, $width) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing window width to Ini file.")
		$ret = 0
	EndIf


	; 2. Write height.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunHeight, $height) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing window height to Ini file.")
		$ret = 0
	EndIf


	; 3. WriteX position.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunXPos, $xpos) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing window X position to Ini file.")
		$ret = 0
	EndIf


	; 4. Write Y position.
	$ret = 1
	If IniWrite($IniWorkFile, $section, $IniRunYPos, $ypos) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing window Y position to Ini file.")
		$ret = 0
	EndIf


	; 5. Exit all done.
	Return($ret)
EndFunc


;
; CreateAutorun()
;
; Entry:		<None>
;Return:		<None>
; Procedure:	1. Warn if already exists.
;			    2. Create file.
;			    3. Exit all done.
;
Func	CreateAutorun()
	Local	$file, $handle, $program, $icon

	; 1. Warn if already exists.
	$file = StringLeft(@ScriptDir, 3)
	$file &= "autorun.inf"
	
	If FileExists($file) <> 0 Then
		If MsgBox(260, $TitleBar2, "Warning - File 'autorun.inf' already exists.  Overwrite it?") <> 6 Then
			Return(0)
		EndIf
	EndIf


	; 2. Create file.
	$handle = FileOpen($file, 2)
	If $handle == -1 Then
		MsgBox(0, $TitleBar2, "Error - Unable to create '" & $file & "'.")
		Return(0)
	EndIf
	FileClose($handle)
	
	$program = @ScriptDir
	If StringRight($program, 1) <> "\" Then $program &= "\"
	$icon = $program & "MultiRun.ico"
	$program &= "MultiRun.exe"	

	If IniWrite($file, "autorun", "OPEN", FileGetShortName($program)) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing 'OPEN' entry to 'autorun.inf' file.")
	EndIf

	If IniWrite($file, "autorun", "ICON", FileGetShortName($icon)) <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing 'ICON' entry to 'autorun.inf' file.")
	EndIf

	If IniWrite($file, "autorun", "ACTION", "Usb-Launcher") <> 1 Then
		MsgBox(0, $TitleBar2, "Error - Failure writing 'ACTION' entry to 'autorun.inf' file.")
	EndIf

	MsgBox(0, $TitleBar2, "Autorun.inf has been created.")


	; 3. Exit all done.
	Return(0)
EndFunc

